/************************************************************************
*      H2O property functions based on:
*          L. Harr, J. Gallagher and G. Kell, "NBS/NRC Steam Tables,
*          Thermodynamic and Transport Properties and Computer Programs
*          for Vapor and Liquid States of Water in SI Units",
*          Hemisphere Publishing Corp., McGraw-Hill International Book
*          Company, 1984, ISBN 0-89116-354-9.
*
*      These functions are low-order fits that are fast running and
*      reasonably accurate (usually < 0.1% error). For more information
*      refer to the following publications:
*
*
*      Please feel free to distribute copies of these functions since the
*      general idea is to promote the free exchange of useful code.
*      Please DO NOT distribute altered copies; the world is confusing
*      enough as it is. I would appreciate being informed of any errors,
*      ommisions, comments, etc.
*
*          Wm. J. Garland
*          Associate Professor
*          Department of Engineering Physics
*          McMaster University
*          1280 Main St. W.
*          Hamilton, Ontario
*          Canada    L8S 4M1
*          Ph: (416) 525-9140 ext. 4925
*
*************************************************************************
*
* Saturation properties based on reference 1 (Garland and Hoskins)
*
* Subcooled and superheated properties based on reference 2 (Garland and
* Hand)
*         SAT means saturated
*         SUB means subcooled
*         SUP means superheated
*
***********************************************************************/

#include <math.h>
#include "h2o.h"
#define  ABS(x) ((x) >= 0 ? (x) : -(x))

/**********************************************************************/
float DF(float P)

/* This function calculates the density of light water
   in liquid phase at saturation for a given pressure.*/

{
      extern char *ERRH2O;
      float DF;

      if (P < 0.002)     {
	     DF=1./(1.9118E-4* pow(0.002,0.546472) + 0.0009947);
	     ERRH2O=" DF: PRESSURE LESS THAN 0.002 MPa    ";
      }
      else if (P > 21.5)     {
	     DF=1./(4.1035988E-24* pow(21.5,15.03329) +0.00189);
	     ERRH2O=" DF: PRESSURE GREATER THAN 21.5 MPa       ";
      }
      else if (P >= 0.002  &&  P < 0.01468)     {
	     DF=1./ (1.9118E-4 * pow(P,0.546472) + 0.0009947);
      }
      else if (P >= 0.01468  &&  P < 0.275)     {
	     DF=1./ (1.380934E-4 * pow(P,0.388715) + 0.000987);
      }
      else if (P >= 0.275 && P <= 1.0)     {
	     DF=1./(1.2746977E-4* pow(P,0.4644339) +0.001);
      }
      else if (P > 1.0 && P <= 3.88)     {
	     DF=1./(1.0476071E-4* pow(P,0.5651090) +0.001022);
      }
      else if (P > 3.88 && P <= 8.84)     {
	     DF=1./(3.2836717E-5* P +1.12174735E-3);
      }
      else if (P > 8.84 && P <= 14.463)     {
	     DF=1./(3.3551046E-4 * exp(0.058403566 *P) +0.00085);
      }
      else if (P > 14.463 && P < 18.052)     {
	     DF= 1./(3.1014626E-8* pow(P,3.284754) +0.00143);
      }
      else if (P >= 18.052 && P < 20.204)     {
	     DF= 1./(1.5490787E-11* pow(P,5.7205) +0.001605);
      }
      else     {
	     DF= 1./(4.1035988E-24* pow(P,15.03329) +0.00189);
      }

      return(DF);

}   /* end of function DF(P) */


/**********************************************************************/
float DFSUB(float P,float T)

/* This function calculates the density of light water
   in the subcooled liquid phase.*/

{
      extern char *ERRH2O;
      float PS,PP,TT,DFSUB;
      float PSAT(float P);
      float DF(float P);

      PS=PSAT(T);
      PP=P;
      TT=T;

      if (P < PS)     {
	   PP=PS;
	   ERRH2O=" DFSUB: PRESSURE LESS THAN SATURATION ";
      }
      else if (P > 18.0)     {
	   PP=18.0;
	   ERRH2O=" DFSUB: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < 17.511)     {
	   TT=17.511;
	   ERRH2O=" DFSUB: TEMPERATURE LESS THAN 17.511 deg C ";
      }
      else if (T > 357.03)     {
	   TT=357.03;
	   ERRH2O=" DFSUB: TEMPERATURE GREATER THAN 357.03 deg C ";
      }

      DFSUB = DF(PS) + (170.0 / (375.0-TT) - 0.2) * ( PP-PS );

      return(DFSUB);

}   /* end of function DFSUB(P,T) */


/**********************************************************************/
float DG(float P)

/* This function calculates the density of light water
   in the gas phase at saturation for a given pressure.*/

{
      extern char *ERRH2O;
      float DG;

      if (P < 0.002)     {
		   DG= 5.0981616 * pow(0.002,0.936226) - 0.00025;
		   ERRH2O=" DG: PRESSURE LESS THAN 0.002 MPa ";
      }
      else if (P > 21.5)     {
		   DG= 3.558734E-16* pow(21.5,13.03774) +138.0;
		   ERRH2O=" DG: PRESSURE GREATER THAN 21.5 MPa ";
      }
      else if (P >= 0.002  &&  P < 0.2139)     {
		   DG= 5.0981616 * pow(P,0.936226) - 0.00025;
      }
      else if (P >= 0.2139 && P < 1.112)     {
		   DG= 5.126076* pow(P,0.9475862) +0.012;
      }
      else if (P >= 1.112 && P < 3.932)     {
		   DG= 4.630832* pow(P,1.038819) +0.52;
      }
      else if (P >= 3.932 && P < 8.996)     {
		   DG= 2.868721* pow(P,1.252148) +3.80;
      }
      else if (P >= 8.996 && P < 14.628)     	
      {
	   	   DG= 0.5497653* pow(P,1.831182) +18.111;
      }
      else if (P >= 14.628 && P <= 18.21)     {
	   	   DG= 8.5791582E-3* pow(P,3.176484) +50.0;
      }
      else if (P > 18.21 && P <= 20.253)     {
	       DG= 3.5587113E-6* pow(P,5.660939) +88.0;
      }
      else     {
	       DG= 3.558734E-16* pow(P,13.03774) +138.0;
      }

      return(DG);

}   /* end of function DG(P) */


/**********************************************************************/
float DGSUP(float P,float T)

/* This function calculates the density of light water
   in the superheated gaseous phase.*/

{
      extern char *ERRH2O;
      float TS,PP,TT,DGSUP;
      float TSAT(float P);
      float VG(float P);

      TS=TSAT(P);
      PP=P;
      TT=T;

      if (P < 0.002)     {
		   PP=0.002;
		   ERRH2O=" DGSUP: PRESSURE LESS THAN 0.002 MPa ";
      }
      else if (P > 18.0)     {
		   PP=18.0;
		   ERRH2O=" DGSUP: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < TS)     {
		   TT=TS;
		   ERRH2O=" DGSUP: TEMPERATURE LESS THAN SATURATION ";
      }
      else if (T > 450.0)     {
		   TT=450.0;
		   ERRH2O=" DGSUP: TEMPERATURE GREATER THAN 450 deg C      ";
      }

           DGSUP = pow(VG(PP)+(0.000466/PP-(0.12/(TT+100.0)-0.00106)
	               *pow(PP,0.1)/sqrt(1.96E-08*pow(TT+8.,4.0)-PP*PP))*(TT-TS),-1.0);

      return(DGSUP);

}   /* end of function DGSUP(P,T) */


/**********************************************************************/
float DHFDP(float P)

/* This function calculates the derivative with respect to
   pressure of the specific enthalpy of light water in the
   liquid phase at saturation for a given pressure.*/

{
      extern char *ERRH2O;
      float DHFDP;

      if (P < 0.002)     {
		   DHFDP= 1128.777*0.135196* pow(0.002,0.135196 -1.0);
		   ERRH2O=" DHFDP: PRESSURE LESS THAN 0.002 MPa    ";
      }
   	  else if (P > 21.7)     {
		   DHFDP= 9.1417257E-14* 11.47287* pow(21.7,10.47287);
		   ERRH2O=" DHFDP: PRESSURE GREATER THAN 21.7 MPa       ";
      }
      else if(P >= 0.002  &&  P < 0.0173)     {
		   DHFDP=1128.777 * 0.135196 * pow(P,(0.135196-1.0));
      }
      else if(P >= 0.0173  &&  P < 0.1028)     {
		   DHFDP=1050.7085 * 0.161797 * pow(P,(0.161797-1.0));
      }
      else if (P >= 0.1028 && P < 0.942)     {
		   DHFDP= 912.1779*0.2061637* pow(P,(0.2061637 -1.0));
      }
      else if (P >= 0.942 && P < 4.02)     {
		   DHFDP= 638.0621*0.2963192* pow(P,(0.2963192 -1.0));
      }
      else if (P >= 4.02 && P < 9.964)     {
		   DHFDP= 373.7665*0.4235532* pow(P,(0.4235532 -1.0));
      }
      else if (P >= 9.964 && P < 16.673)     {
		   DHFDP= 75.38673*0.8282384* pow(P,(0.8282384 -1.0));
      }
      else if (P >= 16.673 && P < 20.396)     {
		   DHFDP= 0.1150827* 2.711412* pow(P,1.711412);
      }
      else    {
		   DHFDP= 9.1417257E-14* 11.47287* pow(P,10.47287);
      }

      return(DHFDP);

}     /* end of function DHFDP (P) */


/**********************************************************************/
float DHFDPT(float P,float T)

/* This function calculates dh/dP |T for light water
   in the subcooled liquid phase.*/

{
      float DHFDPT;

      DHFDPT = 1.4-169.0/(369.0-T);

      return(DHFDPT);

}   /* end of function DHFDPT(P,T) */


/**********************************************************************/
float DHFDTP(float P,float T)

/* This function calculates dh/dT |P for light water
   in the subcooled liquid phase.*/

{
      float PS,DHFDTP;
      float PSAT(float P);
      float DPDT(float P);
      float DHFDP(float PS);

      PS=PSAT(T);

      DHFDTP = -169.0/((369.0-T)*(369.0-T))*(P-PS)+DPDT(T)
	       * (DHFDP(PS)-1.4+169.0/(369.0-T));

      return(DHFDTP);

}   /* end of function DHFDTP(P,T) */


/**********************************************************************/
float DHGDP(float P)

/* This function calculates the derivative with respect to
   pressure of the specific enthalpy of light water in the gas
   phase at saturation for a given pressure.*/
{
      extern char *ERRH2O;
      float DHGDP;

      if (P < 0.002)     {
		   DHGDP= 529.44008 * 0.108652 * pow(0.002,0.108652-1.0);
		   ERRH2O=" DHGDP: PRESSURE LESS THAN 0.002 MPa    ";
      }
      else if (P > 21.55)     {
		   DHGDP= -22.92521*2.0* (21.55-18.0) +44.23671;
		   ERRH2O=" DHGDP: PRESSURE GREATER THAN 21.55 MPa       ";
      }
      else if(P >= 0.002  &&  P < 0.1379)     {
		   DHGDP= 529.44008 * 0.108652 * pow(P,(0.108652-1.0));
      }
      else if (P >= 0.1379 && P <= 0.348)     {
		   DHGDP= 4.0381938E-6*15.72364* pow(3.0-P,14.72364);
      }
      else if (P > 0.348 && P <= 1.248)     {
		   DHGDP= 0.5767304*1.66153* exp(-1.66153* (P-3.2));
      }
      else if (P > 1.248 && P < 2.955)     {
		   DHGDP= 7.835986*2.0* (3.001-P) -2.934312;
      }
      else if (P >= 2.955 && P <= 6.522)     {
		   DHGDP= -1.347244*2.0* (P-2.999) -2.326913;
      }
      else if (P > 6.522 && P < 16.497)     {
		   DHGDP= -0.9219176*2.0* (P-9.0) -16.38835;
      }
      else if (P >= 16.497 && P < 20.193)     {
		   DHGDP= -3.432177*2.0* (P-8.0) +29.81305;
      }
      else    {
		   DHGDP= -22.92521*2.0* (P-18.0) +44.23671;
      }

      return(DHGDP);

}   /* end of function DHGDP(P) */


/**********************************************************************/
float DHGDPT(float P,float T)

/* This function calculates dh/dP |T for light water
   in the superheated gaseous phase.*/

{
      float TS,PS,PTEMP,DHGDPT;
      float TSAT(float P);
      float PSAT(float T);
      float DTDP(float P);

      TS=TSAT(P);
      PS=PSAT(T);
      PTEMP=P;
      P = P + 0.13 * (PS-P);

      DHGDPT=DHGDP(P)-(4.5*P/sqrt(7.4529E-06*T*T*T-P*P)
		    +0.28*exp(-0.008*(T-162.0))-100.0/T+2.225)*DTDP(P)+(T-TS)
		    * (4.5/sqrt(7.4529E-06*T*T*T-P*P)
		    +(4.5*P*P/pow(7.4529E-06*T*T*T-P*P,1.5)));

      P=PTEMP;

      return(DHGDPT);

}   /* end of function DHGDPT(P,T) */


/**********************************************************************/
float DHGDTP(float P,float T)

/* This function calculates dh/dT |P for light water
   in the superheated gaseous phase.*/

{
      float TS,DHGDTP;
      float TSAT(float P);

      TS=TSAT(P);

      DHGDTP = +(4.5*P/sqrt(7.4529E-06*T*T*T-P*P)
	     +0.28*exp(-0.008*(T-162.0))-100.0/T+2.225)
	     + (-(4.5*P/pow(7.4529E-06*T*T*T-P*P,1.5))*1.5*T*T*7.4529E-06
	     - 0.28*0.008*exp(-0.008*(T-162.0))+100.0/T/T )*(T-TS);

      return(DHGDTP);

}   /* end of function DHGDTP(P,T) */


/**********************************************************************/
float DPDT(float T)

/* This function calculates dP/dT along the saturation line
   for a given temperature.*/

{
      extern char *ERRH2O;
      float DPDT;

      if (T < 17.511)     {
		   DPDT=7.406365/270.121*pow((17.511+99.2)/270.121,7.406365-1.0);
		   ERRH2O=" DPDT: TEMPERATURE LESS THAN 17.511 deg C    ";
      }
      else if (T > 373.253)     {
		   DPDT=4.960785/227.2963*pow((373.253+50.0)/227.2963,4.960785-1.0);
		   ERRH2O=" DPDT: TEMPERATURE GREATER THAN 373.253 deg C       ";
      }
      else if(T >= 17.511  &&  T < 56.275)     {
		   DPDT=7.406365/270.121*pow((T+99.2)/270.121,7.406365-1.0);
      }
      else if(T >= 56.275  &&  T < 90.88)     {
		   DPDT=6.4058216/254.6831*pow((T+78.2)/254.6831,6.4058216-1.0);
      }
      else if (T >= 90.88 && T < 139.781)     {
		   DPDT=5.602972/236.2315*pow((T+57.0)/236.2315,5.602972-1.0);
      }
      else if (T > 139.781 && T < 203.662)     {
		   DPDT=4.778504/207.9248*pow((T+28.0)/207.9248,4.778504-1.0);
      }
      else if (T >= 203.662 && T <= 299.407)     {
		   DPDT=4.304376/185.0779*pow((T+5.0)/185.0779,4.304376-1.0);
      }
      else if (T > 299.407 && T < 355.636)     {
		   DPDT=4.460843/195.1819*pow((T+16.0)/195.1819,4.460843-1.0);
      }
      else   {
		   DPDT=4.960785/227.2963*pow((T+50.0)/227.2963,4.960785-1.0);
      }

      return(DPDT);

}   /* end of function DPDT(T) */


/**********************************************************************/
float DRFDPT(float P,float T)

/* This function calculates dRHO/dP |T for light water
   in the subcooled liquid phase.*/

{
      float DRFDPT;

      DRFDPT = 170.0/(375.0-T)-0.2;

      return(DRFDPT);

}   /* end of function DRFDPT(P,T) */


/**********************************************************************/
float DRFDTP(float P,float T)

/* This function calculates dRHO/dT |P for light water
   in the subcooled liquid phase.*/

{
     float PS,DRFDTP;
     float PSAT(float T);
     float DPDT(float T);
     float DVFDP(float P);
     float DF(float PS);

     PS=PSAT(T);

     DRFDTP = (170.0/(375.0-T)/(375.0-T))*(P-PS)+DPDT(T)
	    * (-DF(PS)*DF(PS)*DVFDP(PS)-170.0/(375.0-T)+0.2);

     return(DRFDTP);

}   /* end of function DRFDTP(P,T) */


/**********************************************************************/
float DRGDPT(float P,float T)

/* This function calculates dRHO/dP |T for light water
   in the superheated gaseous phase.*/

{
     float TS,DVGDPT,DRGDPT;
     float TSAT(float P);
     float DVGDP(float P);
     float DTDP(float P);
     float DGSUP(float P,float T);

     TS=TSAT(P);

     DVGDPT = DVGDP(P)-(0.000466/P-(0.12/(T+100.0)-0.00106)*
		    pow(P,0.1)/sqrt(1.96E-08*pow(T+8.,4.0)-P*P))*
		    DTDP(P)+(-0.000466/P/P-(0.12/(T+100.0)-0.00106)*0.1*
		    pow(P,0.1-1.0)/sqrt(1.96E-08*pow(T+8.,4.0)-P*P)-((0.12/(T+100.0)-
		    0.00106)*pow(P,1.1)/pow(1.96E-8*pow(T+8,4.0)-P*P,1.5)))*(T-TS);

     DRGDPT=-DGSUP(P,T)*DGSUP(P,T)*DVGDPT;

     return(DRGDPT);

}  /* end of function DRGDPT(P,T) */


/**********************************************************************/
float DRGDTP(float P,float T)

/* This function calculates dRHO/dT |P for light water
   in the superheated gaseous phase.*/

{
     float TS,DRGDTP,DVGDTP;
     float TSAT(float P);
     float DGSUP(float P,float T);

     TS=TSAT(P);

     DVGDTP =  0.000466/P-( 0.12 / (T+100.0) - 0.00106 )*pow(P,0.1)
		    / sqrt ( 1.96E-08 * pow(T+8.0,4.0) -P*P) + (T-TS)*
		    ( ( 0.12 / pow(T+100.0,2))*pow(P,0.1)
		    / sqrt ( 1.96E-08 * pow(T+8.0,4.0) -P*P) + (0.12 / (T+100.0)
		    - 0.00106 )*pow(P,0.1) / pow( 1.96E-08 * pow(T+8.0,4.0) -P*P,1.5)
		    /2 * 1.96E-08 * 4.0 * pow(T+8.0,3.0));

     DRGDTP=-DGSUP(P,T)*DGSUP(P,T)*DVGDTP;

     return(DRGDTP);

}   /* end of function DRGDTP(P,T) */

/**********************************************************************/
float DTDP(float P)

/* This function calculates dT/dP along the saturation line
   for a given pressure.*/

{
     extern char *ERRH2O;
     float DTDP;

     if (P < 0.002)     {
		  DTDP=0.1350197 * 270.121* pow(0.002,0.135019 -1.0);
		  ERRH2O=" DTDP: PRESSURE LESS THAN 0.002 MPa ";
     }
     else if (P > 21.85)     {
		  DTDP= 0.201581 * 227.2963* pow(21.85,0.201581 - 1.0);
		  ERRH2O=" DTDP: PRESSURE GREATER THAN 21.85 MPa ";
     }
     else if (P >= 0.002  &&  P < 0.01672)     {
		  DTDP=0.135019 * 270.121 * pow(P,0.135019-1.0);
     }
     else if (P >= 0.01672 && P < 0.0725)     {
		  DTDP=0.156108 * 254.6831 * pow(P,0.156108-1.0);
     }
     else if (P >= 0.0725 && P <= 0.359)     {
		  DTDP=0.1784767 * 236.2315* pow(P,0.1784767 -1.0);
     }
     else if (P > 0.359 && P <= 1.676)     {
		  DTDP=0.2092705 * 207.9248* pow(P,0.2092705 -1.0);
     }
     else if (P > 1.676 && P <= 8.511)     {
		  DTDP=0.2323217 * 185.0779* pow(P,0.2323217 -1.0);
     }
     else if (P > 8.511 && P <= 17.69)     {
		  DTDP=0.2241729 * 195.1819* pow(P,0.2241729 -1.0);
     }
     else       {
		  DTDP= 0.201581 * 227.2963*pow(P,0.201581 - 1.0);
     }

     return(DTDP);

}   /* end of function DTDP(P) */


/**********************************************************************/
float DVFDP(float P)

/* This function calculates the derivative with respect to
   pressure of the specific volume of light water in the liquid
   phase at saturation for a given pressure.*/

{
     extern char *ERRH2O;
     float DVFDP;

     if (P < 0.002)     {
		   DVFDP=1.9118E-4*0.546472* pow(0.002,0.546472-1.0);
		   ERRH2O=" DVFDP: PRESSURE LESS THAN 0.002 MPa ";
     }
     else if (P > 21.5)     {
		   DVFDP= 4.1035988E-24* 15.03329* pow(21.5,14.03329);
		   ERRH2O=" DVFDP: PRESSURE GREATER THAN 21.5 MPa ";
     }
     else if (P >= 0.002  &&  P < 0.01468)     {
		   DVFDP=1.9118E-4 * 0.546472 * pow(P,0.546472-1.0);
     }
     else if (P >= 0.01468  &&  P < 0.275)     {
		   DVFDP=1.380934E-4 * 0.388715 * pow(P,0.388715-1.0);
     }
     else if (P >= 0.275 && P <= 1.0)     {
		   DVFDP=1.2746977E-4*0.4644339* pow(P,0.4644339-1.0);
     }
     else if (P > 1.0 && P <= 3.88)     {
		   DVFDP=1.0476071E-4*0.5651090* pow(P,0.5651090 -1.0);
     }
     else if (P > 3.88 && P <= 8.84)     {
		   DVFDP= 3.2836717E-5;
     }
     else if (P > 8.84 && P <= 14.463)     {
		   DVFDP=3.3551046E-4*0.058403566* exp(0.058403566*P);
     }
     else if (P > 14.463 && P < 18.052)     {
		   DVFDP= 3.1014626E-8* 3.284754* pow(P,2.284754);
     }
     else if (P >= 18.052 && P < 20.204)     {
		   DVFDP= 1.5490787E-11* 5.7205* pow(P,4.7205);
     }
     else   {
		   DVFDP= 4.1035988E-24* 15.03329* pow(P,14.03329);
     }

     return(DVFDP);

}   /* end of function DVFDP(P) */

float DVGDP(float P)

/* This function calculates the derivative with respect to
   pressure of the specific volume of light water in the gas
   phase at saturation for a given pressure.*/

{
     extern char *ERRH2O;
     float DVGDP,DDGDP;
     float VG(float P);

     if (P < 0.002)     {
		   DDGDP= 5.0981616 * 0.936226 * pow(0.002,0.936226-1.0);
		   DVGDP=-1.0* VG(0.002)*VG(0.002) *DDGDP;
		   ERRH2O=" DVGDP: PRESSURE LESS THAN 0.002 MPa ";
     }
     else if (P > 21.5)     {
		   DDGDP= 3.558734E-16* 13.03774* pow(21.5,12.03774);
		   DVGDP=-1.0* VG(21.5)*VG(21.5) *DDGDP;
		   ERRH2O=" DVGDP: PRESSURE GREATER THAN 21.5 MPa ";
     }
     else if (P >= 0.002  &&  P < 0.2139)     {
		   DDGDP= 5.0981616 * 0.936226 * pow(P,0.936226-1.0);
		   DVGDP=-1.0 * VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 0.2139 && P < 1.112)     {
		   DDGDP= 5.126076*0.9475862* pow(P,0.9475862 -1.0);
		   DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 1.112 && P < 3.932)     {
		   DDGDP= 4.630832*1.038819* pow(P,0.038819);
		   DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 3.932 && P < 8.996)     {
		   DDGDP= 2.868721*1.252148* pow(P,0.252148);
		   DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 8.996 && P < 14.628)     {
		   DDGDP= 0.5497653*1.831182* pow(P,0.831182);
		   DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 14.628 && P <= 18.21)     {
		   DDGDP= 8.5791582E-3* 3.176484* pow(P,2.176484);
		   DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P > 18.21 && P <= 20.253)     {
		   DDGDP= 3.5587113E-6* 5.660939* pow(P,4.660939);
		   DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else    {
		   DDGDP= 3.558734E-16* 13.03774* pow(P,12.03774);
		   DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }

     return(DVGDP);

}   /* end of function DVGDP(P) */


/**********************************************************************/
float HF(float P)

/* This function calculates the specific enthalpy of light water
   in the liquid phase at saturation for a given pressure.*/

{
     extern char *ERRH2O;
     float HF;

     if (P < 0.002)     {
		   HF= 1128.777 * pow(0.002,0.135196) - 413.72;
		   ERRH2O=" HF: PRESSURE LESS THAN 0.002 MPa ";
     }
     else if (P > 21.7)     {
		   HF= 9.1417257E-14* pow(21.7,11.47287) +1752.0;
		   ERRH2O=" HF: PRESSURE GREATER THAN 21.7 MPa ";
     }
     else if (P >= 0.002  &&  P < 0.0173)     {
		   HF= 1128.777 * pow(P,0.135196) - 413.72;
     }
     else if (P >= 0.0173  &&  P < 0.1028)     {
		   HF= 1050.7085 * pow(P,0.161797) - 306.5;
     }
     else if (P >= 0.1028 && P < 0.942)     {
		   HF= 912.1779* pow(P,0.2061637) -150.0;
     }
     else if (P >= 0.942 && P < 4.02)     {
		   HF= 638.0621* pow(P,0.2963192) +125.0;
     }
     else if (P >= 4.02 && P < 9.964)     {
		   HF= 373.7665* pow(P,0.4235532) +415.0;
     }
     else if (P >= 9.964 && P < 16.673)     {
		   HF= 75.38673* pow(P,0.8282384) +900.0;
     }
     else if (P >= 16.673 && P < 20.396)     {
		   HF= 0.1150827* pow(P,2.711412) +1440.0;
     }
     else     {
		   HF= 9.1417257E-14* pow(P,11.47287) +1752.0;
     }

     return(HF);

}   /* end of function HF(P) */


/**********************************************************************/
float HFSUB(float P,float T)

/* This function calculates the enthalpy of light water
   in the subcooled liquid phase.*/

{
      extern char *ERRH2O;
      float PP,TT,PS,HFSUB;
      float PSAT(float T);
      float HF(float P);

      PP=P;
      TT=T;
      PS=PSAT(T);

      if (P < PS)     {
		    PP=PS;
		    ERRH2O=" HFSUB: PRESSURE LESS THAN SATURATION ";
      }
   	  else if (P > 18.)     {
		    PP=18.;
		    ERRH2O=" HFSUB: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < 17.511)     {
		    TT=17.511;
		    ERRH2O=" HFSUB: TEMPERATURE LESS THAN 17.511 deg C ";
      }
      else if (T > 357.03)     {
		    TT=357.03;
		    ERRH2O=" HFSUB: TEMPERATURE GREATER THAN 357.03 deg C ";
      }

      HFSUB = HF(PS) + (1.4-169./(369.-TT))*(PP-PS);

      return(HFSUB);

}   /* end of function HFSUB(P,T) */


/**********************************************************************/
float HG(float P)

/* This function calculates the specific enthalpy of light water
   in the gas phase at saturation for a given pressure.*/

{
      extern char *ERRH2O;
      float HG;

      if (P < 0.002)     {
		    HG= 529.44008 * pow(0.002,0.108652) + 2263.5;
		    ERRH2O=" HG: PRESSURE LESS THAN 0.075 MPa ";
      }
      else if (P > 21.55)     {
		    HG= -22.92521* pow(21.55-18.0,2.0) +44.23671* (21.55-18.0) + 2415.01;
		    ERRH2O=" HG: PRESSURE GREATER THAN 21.55 MPa ";
      }
      else if (P >= 0.002  &&  P < 0.1379)     {
		    HG= 529.44008 * pow(P,0.108652) + 2263.5;
      }
      else if (P >= 0.1379 && P <= 0.348)     {
		    HG= -4.0381938E-6* pow(3.0-P,15.72364) +2750.0;
      }
      else if (P > 0.348 && P <= 1.248)     {
		    HG= -0.5767304* exp(-1.66153* (P-3.2)) +2800.0;
      }
      else if (P > 1.248 && P < 2.955)     {
		    HG= -7.835986* (3.001-P)*(3.001-P) +2.934312*(3.001-P) +2803.71;
      }
      else if (P >= 2.955 && P <= 6.522)     {
		    HG=-1.347244*(P-2.999)*(P-2.999)-2.326913*(P-2.999)+2803.35;
      }
      else if (P > 6.522 && P < 16.497)     {
		    HG= -0.9219176* (P-9.0)*(P-9.0) -16.38835*(P-9.0) +2742.03;
      }
      else if (P >= 16.497 && P < 20.193)     {
		    HG= -3.532177* (P-8.0)*(P-8.0) +29.81305* (P-8.0) +2565.00;
      }
      else     {
		    HG= -22.92521* (P-18.0)*(P-18.0) +44.23671* (P-18.0) +2415.01;
      }

      return(HG);

}   /* end of function HG(P) */


/**********************************************************************/
float HGSUP(float P,float T)

/* This function calculates the enthalpy of light water
   in the superheated gaseous phase.*/

{
      extern char *ERRH2O;
      float TS,PP,TT,HGSUP;
      float TSAT(float P);
      float HG(float P);

      TS=TSAT(P);
      PP=P;
      TT=T;

      if (P < 0.002)     {
		    PP=0.002;
		    ERRH2O=" HGSUB: PRESSURE LESS THAN 0.002 MPa ";
      }
      else if (P > 18.0)     {
		    PP=18.0;
		    ERRH2O=" HGSUB: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < TS)     {
		    TT=TS;
		    ERRH2O=" HGSUB: TEMPERATURE LESS THAN SATURATION ";
      }
      else if (T > 450.0)     {
		    TT=450.0;
		    ERRH2O=" HGSUB: TEMPERATURE GREATER THAN 450 deg C ";
      }

      HGSUP = HG(PP) +(4.5*PP/sqrt(7.4529E-06*pow(TT,3.0)-PP*PP)+0.28
		    * exp(-0.008*(TT-162.0))-100.0/TT+2.225)*(TT-TS);

      return(HGSUP);

}   /* end of function HGSUP(P,T) */


/**********************************************************************/
float PSAT(float T)

/* This function calculates the saturation pressure of light water
   for a given temperature.*/

{
      extern char *ERRH2O;
      float PSAT;

      if (T < 17.511)     {
		    PSAT=pow((17.511+99.2)/270.121,7.406365);
		    ERRH2O=" PSAT: TEMPERATURE LESS THAN 17.511 deg C    ";
      }
      else if (T > 373.253)     {
		    PSAT=pow((373.253+50.0)/227.2963,4.960785);
		    ERRH2O=" PSAT: TEMPERATURE GREATER THAN 373.253 deg C       ";
      }
      else if (T >= 17.511  &&  T < 56.275)     {
		    PSAT=pow((T+99.2)/270.121,7.406365);
      }
      else if (T >= 56.275  &&  T < 90.88)     {
		    PSAT=pow((T+78.2)/254.6831,6.4058216);
      }
      else if (T >= 90.88 && T < 139.781)     {
		    PSAT=pow((T+57.0)/236.2315,5.602972);
      }
      else if (T > 139.781 && T < 203.662)     {
		    PSAT=pow((T+28.0)/207.9248,4.778504);
      }
      else if (T >= 203.662 && T <= 299.407)     {
		    PSAT=pow((T+5.0)/185.0779,4.304376);
      }
      else if (T > 299.407 && T < 355.636)     {
		    PSAT=pow((T+16.0)/195.1819,4.460843);
      }
      else       {
		    PSAT=pow((T+50.0)/227.2963,4.960785);
      }

      return(PSAT);

}   /* end of function PSAT(T) */


/**********************************************************************/
float TSAT(float P)

/* This function calculates the saturation temperature of light water for
   a given pressure.*/

{
	  extern char *ERRH2O;
	  float TSAT;

	  if (P < 0.002)     {
			 TSAT=270.121* pow(0.002,0.135019) - 99.2;
             ERRH2O=" TSAT: PRESSURE LESS THAN 0.002 MPa ";
	  }
      else if (P > 21.85)     {
			 TSAT= 227.2963* pow(21.85,0.201581) - 50.0;
             ERRH2O=" TSAT: PRESSURE GREATER THAN 21.85 MPa ";
	  }
	  else if (P >= 0.002  &&  P < 0.01672)     {
			 TSAT=270.121 * pow(P,0.135019) - 99.2;
	  }
	  else if (P >= 0.01672  &&  P < 0.0725)     {
			 TSAT=254.6831 * pow(P,0.156108) - 78.2;
	  }
      else if (P >= 0.0725 && P <= 0.359)     {
			 TSAT=236.2315* pow(P,0.1784767) -57.0;
	  }
      else if (P > 0.359 && P <= 1.676)     {
			 TSAT=207.9248* pow(P,0.2092705) -28.0;
	  }
      else if (P > 1.676 && P <= 8.511)     {
			 TSAT=185.0779* pow(P,0.2323217) -5.0;
	  }
      else if (P > 8.511 && P <= 17.69)     {
			 TSAT=195.1819* pow(P,0.2241729) - 16.0;
	  }
      else       {
			 TSAT=227.2963* pow(P,0.201581) - 50.0;
	  }

	  return(TSAT);

}   /* end of function TSAT(P) */


/**********************************************************************/
float VF(float P)

/* This function calculates the specific volume of light water
   in liquid phase at saturation for a given pressure.*/

{
	  extern char *ERRH2O;
	  float VF;

	  if (P < 0.002)     {
			 VF=1.9118E-4 * pow(0.002,0.546472) + 0.0009947;
             ERRH2O=" VF: PRESSURE LESS THAN 0.002 MPa ";
	  }
      else if (P > 21.5)     {
			 VF= 4.1035988E-24* pow(21.5,15.03329) +0.00189;
             ERRH2O=" VF: PRESSURE GREATER THAN 21.5 MPa ";
	  }
	  else if (P >= 0.002  &&  P < 0.01468)     {
			 VF=1.9118E-4 * pow(P,0.546472) + 0.0009947;
	  }
	  else if (P >= 0.01468  &&  P < 0.275)     {
			 VF=1.380934E-4 * pow(P,0.388715) + 0.000987;
	  }
      else if (P >= 0.275 && P <= 1.0)     {
			 VF=1.2746977E-4* pow(P,0.4644339) +0.001;
	  }
      else if (P > 1.0 && P <= 3.88)     {
			 VF=1.0476071E-4* pow(P,0.5651090) +0.001022;
	  }
      else if (P > 3.88 && P <= 8.84)     {
             VF=3.2836717E-5* P +1.12174735E-3;
	  }
      else if (P > 8.84 && P <= 14.463)     {
             VF=3.3551046E-4 * exp(0.058403566 *P) +0.00085;
	  }
	  else if (P > 14.463 && P < 18.052)     {
			 VF= 3.1014626E-8* pow(P,3.284754) +0.00143;
	  }
	  else if (P >= 18.052 && P < 20.204)     {
			 VF= 1.5490787E-11* pow(P,5.7205) +0.001605;
	  }
      else     {
			 VF= 4.1035988E-24* pow(P,15.03329) +0.00189;
	  }

	  return(VF);

}   /* end of function VF(P) */


/**********************************************************************/
float VFSUB(float P,float T)

/* This function calculates the specific volume of light water
   in the subcooled liquid phase.*/

{
	  extern char *ERRH2O;
	  float PS,PP,TT,VFSUB;
	  float PSAT(float T);
	  float DF(float P);

	  PS=PSAT(T);
      PP=P;
      TT=T;

	  if (P < PS)     {
             PP=PS;
             ERRH2O=" VFSUB: PRESSURE LESS THAN SATURATION ";
	  }
	  else if (P > 18.0)     {
			 PP=18.0;
             ERRH2O=" VFSUB: PRESSURE GREATER THAN 18 MPa ";
	  }
	  else if (T < 17.511)     {
             TT=17.511;
             ERRH2O=" VFSUB: TEMPERATURE LESS THAN 17.511 deg C ";
	  }
      else if (T > 357.03)     {
             TT=357.03;
             ERRH2O=" VFSUB: TEMPERATURE GREATER THAN 357.03 deg C ";
	  }

	  VFSUB = pow(DF(PS) + (170.0 / (375.0-TT) - 0.2) * (PP-PS ),-1.0);

	  return(VFSUB);

}   /* end of function VFSUB(P,T) */


/**********************************************************************/
float VG(float P)

/* This function calculates the specific volume of light water
   in the gas phase at saturation for a given pressure.*/

{
	  extern char *ERRH2O;
	  float VG;

	  if (P < 0.002)     {
			 VG= 1.0/(5.0981616 * pow(0.002,0.936226) -0.00025);
             ERRH2O=" VG: PRESSURE LESS THAN 0.085 MPa ";
	  }
      else if (P > 21.5)     {
			 VG= 1.0/( 3.558734E-16* pow(21.5,13.03774) +138.0);
             ERRH2O=" VG: PRESSURE GREATER THAN 21.5 MPa ";
	  }
	  else if (P >= 0.002  &&  P < 0.2139)     {
			 VG= 1.0/ (5.0981616 * pow(P,0.936226) -0.00025);
	  }
	  else if (P >= 0.2139 && P < 1.112)     {
			 VG= 1.0/(5.126076* pow(P,0.9475862) +0.012);
	  }
	  else if (P >= 1.112 && P < 3.932)     {
			 VG= 1.0/(4.630832* pow(P,1.038819) +0.52);
	  }
	  else if (P >= 3.932 && P < 8.996)     {
			 VG= 1.0/(2.868721* pow(P,1.252148) +3.80);
	  }
	  else if (P >= 8.996 && P < 14.628)     {
			 VG= 1.0/(0.5497653* pow(P,1.831182) +18.111);
	  }
      else if (P >= 14.628 && P <= 18.21)     {
			 VG=1.0/(8.5791582E-3* pow(P,3.176484) +50.0);
	  }
      else if (P > 18.21 && P <= 20.253)     {
			 VG= 1.0/( 3.5587113E-6* pow(P,5.660939) +88.0);
	  }
      else     {
			 VG= 1.0/( 3.558734E-16* pow(P,13.03774) +138.0);
	  }

	  return(VG);

}   /* end of function VG(P) */


/**********************************************************************/
float VGSUP(float P,float T)

/* This function calculates the specific volume of light water
   in the superheated gaseous phase.*/

{
	  extern char *ERRH2O;
	  float TS,PP,TT,VGSUP;
	  float TSAT(float P);
	  float VG(float P);

	  TS=TSAT(P);
      PP=P;
      TT=T;

	  if (P < 0.002)     {
             PP=0.002;
             ERRH2O=" VGSUP: PRESSURE LESS THAN 0.002 MPa ";
	  }
	  else if (P > 18.0)     {
			 PP=18.0;
             ERRH2O=" VGSUP: PRESSURE GREATER THAN 18 MPa ";
	  }
	  else if (T < TS)     {
             TT=TS;
             ERRH2O=" VGSUP: TEMPERATURE LESS THAN SATURATION ";
	  }
	  else if (T > 450.0)     {
			 TT=450.0;
             ERRH2O=" VGSUP: TEMPERATURE GREATER THAN 450 deg C ";
	  }

	  VGSUP = VG(PP)+(0.000466/P-(0.12/(TT+100.0) - 0.00106 )*pow(PP,0.1)
			/ sqrt ( 1.96E-08 * pow(TT+8.0,4.0) -PP*PP)) * (TT-TS);

	  return(VGSUP);

}   /* end of function VGSUP(P,T) */


/**********************************************************************/
float VISCF(float P)

/* dynamic viscosity - saturated liquid */

{
	  float VFP,TS;
	  float VF(float P);
	  float TSAT(float P);
	  float visco(float v,float T);

	  VFP = VF(P);
	  TS  = TSAT(P);

	  return (visco(VFP,TS));


}   /* end of function VISCF(P,T) */


/**********************************************************************/
float VIFSUB(float P,float T)

/* dynamic viscosity - subcooled liquid */

{
	  float VFP;
	  float VFSUB(float P,float T);
	  float visco(float v,float T);

	  VFP = VFSUB(P,T);

	  return(visco(VFP,T));


}   /* end of function VIFSUB(P,T) */


/**********************************************************************/
float VISCG(float P)

/* dynamic viscosity - saturated vapor */

{
	  float VGP,TS;
	  float VG(float P);
	  float TSAT(float P);
	  float visco(float v,float T);

	  VGP = VG(P);
	  TS  = TSAT(P);

	  return (visco(VGP,TS));

}   /* end of function VISCG(P,T) */


/**********************************************************************/
float VIGSUP(float P,float T)

/* dynamic viscosity - superheated vapor  */

{
	  float VGP;
	  float VGSUP(float P,float T);
	  float visco(float v,float T);

	  VGP = VGSUP(P,T);

	  return(visco(VGP,T));

}   /* end of function VIGSUP(P,T) */


/**********************************************************************/
float visco(float vol,float T)

/* dynamic viscosity of water and steam - IAPS '75 formulation
                                           for scientific purposes

			input : vol   - specific volume (m^3/kg)
			         T     - tmperature (0C)	
			output: visco - dynamic viscosity (Pa*s*10^-6) */

{
	  float tred,rhored,xmi,tb,rob,a,b,c,d,e,xm,rm,r0,tk,dmi,visco;

	  tred = (T + 273.15) / 647.27;
	  rhored = 1.0/ vol / 317.763;
	  xmi = 100.0*sqrt(tred)/(((-0.36744/tred+1.05287)/tred+1.77624)/tred+1.81583);
	  tb = 1.0/tred-1.0;
	  rob = rhored-1.0;
	  a = ((((0.146543 * tb-0.551119) * tb+0.907919) * tb-0.130356) * tb+0.162888) * tb+0.501938;
	  b = ((((-0.084337 * tb+0.0670665) * tb+1.207552) * tb+0.6736651) * tb+0.789393)*tb+0.235622;
	  c = ((((0.195286 * tb-0.497089) * tb-0.687343) * tb-0.959456) * tb-0.743539) * tb-0.274637;
	  d = ((((-0.032932 * tb+0.100754) * tb+0.213486) * tb+0.347247) * tb+0.263129) * tb+0.145831;
	  e = ((((-0.0202595 * tb+0.0602253) * tb-0.0822904) *tb-0.0267758)* tb-0.0253093) * tb-0.0270448;
      xm = (((e*rob+d)*rob+c)*rob+b)*rob+a;
      xm = xmi*exp(rhored*xm);

	  if (T >= 367.0  &&  T <= 380.0) {
		 rm = 317.763+107.661*pow(ABS(tred-1.0),0.067174)*exp(-543.48*ABS(tred-1.0));
         r0 = sqrt(3.88937+5.11063/tred);
		 tk = 1.0/ABS(vol*rm*tred*pow(r0/(r0-1.0/vol/rm),2)-2.25);
		 dmi = 0.01204*xmi*tk*exp(-102.4*pow(rob,4.0)-5279.*pow(tred-1.0,2.0));
		 dmi = dmi+0.1432*xmi*exp(-1107.*pow(rob,4.0)-3.08e5*pow(tred-1.0,2.0));
         visco = xm + dmi;
	  }
	  else                          {
         visco = xm;
	  }

	  return(visco);

}   /* end of function visco(VOL,T) */


