/* simulator.c - a thermalhydraulic simulator using the homogeneous equilibrium model */

#include "simulat.h"  /* function prototypes */
#include "nrutil.h"		/* numerical recipes utilities function prototypes */
#include "nr.h"			/* numerical recipes function prototypes */
#include "H2O.h"		/* water function prototypes and extern variable declarations */
#include "file.h"		/* file handling function prototypes */
#include <stdio.h>
#include <time.h>

int NN,NL;

main()
{	
	/* node variable vectors */

	float *mass,*enth,*spech,*vol,*temp,*press,*heat,*rho;
	float *dpdt,*dmdt,*dhdt,*dtempdt;
	float *hold,*G1temp,*G2temp;
	float *c1,*c2,*voi,*qual;
	
	/* link variable vectors */
	
	float *k,*loss;
	float *w,*den,*work;
	int *iup,*idown,*ifrom;
	float *dia,*len,*dz;
	float *dppump,*area;

	/* matrices */
	
	float **mwmat,**hwmat,**pwmat,**wpmat,**wwmat;
	float *bwmat,*bhmat,*bpmat,*bmmat;
	float **term1,*term2,**matran;
	
	/* misc constants */
	
	float grav=9.81;
	
	/* time, time step, and print time variables */
	
	float t=0.0,dt=0.0,tprint=0.0;
	float tfin,dtmin,dtmax,ts,tp,terr,pscale,adj,d,dtlast=0.0;
	
	/* misc status integers, counters, limits */
	
	int ifull,nmax,i,j,*indx;
	int isat=6;
	int flag=0;
	
	/* variables for run time determination */

	time_t start,finish;
	double totaltime;
	
	/* file pointers */
	
	FILE *infile,*outfile;
	
	
	/* get time at beginning of program for run time calculations and initialize
	   code profiler */	
	
	/*InitProfile(200,200);		*/
	start=time(&start);


/************************ file stuff and vector/matrix declarations *******************/
	
	/* read in the number of nodes and links in the hydraulic network */
	
	READ_NN_NL(&infile);
		
	/* declare vector and matrix sizes using NN and NL and set values to zero; three
	   declare functions are required due to number of formal parameters which must be
	   passed.  These declare functions use modified versions of the numerical recipes
	   in C vector and matrix functions.  Instead of using MALLOC, CALLOC is used in
	   the NR utilities to initialize the vectors to zero */
	
	DECLARE1(&mass,&enth,&spech,&vol,&temp,&press,&heat,&rho,&dpdt,&dmdt,&dhdt,&dtempdt,
			&hold,&G1temp,&G2temp,&c1,&c2,&voi,&qual);
	DECLARE2(&area,&len,&dia,&dz,&dppump,&k,&loss,&w,&den,&work,&bwmat,&bhmat,&bpmat,
			&bmmat,&term2);
	DECLARE3(&indx,&iup,&idown,&ifrom,&mwmat,&hwmat,&pwmat,&wpmat,&wwmat,&matran,
			&term1);
	
	/* read data from the input file */
	
	READ_INPUT(infile,&tfin,&dtmin,&dtmax,&ts,&tp,&terr,&pscale,&adj,&ifull,&nmax,vol,
			press,spech,area,len,dia,dz,loss,w,iup,idown,ifrom);

	/* open up the output file and write header */
	
	INIT_OUTPUT(&outfile,ifull);
	

/************************ start of computational section ******************************/	
	
	/* an initial guess for temperature is made, close to saturation, this guess
	   will be corrected when GET_H2O_PROPS is called */
	   
	for(i=1;i<=NN;i++)
		temp[i]=0.9*TSAT(press[i]);
		
	
	/* calculate initial enthalpies, densities, and masses using GET_H2O_PROPS routine;
	   isat was previously set to 6 to indicate that property determination is to be
	   on the basis of known pressure and specific enthalpy */
	
	GET_H2O_PROPS(spech,mass,vol,press,temp,c1,c2,enth,rho,voi,qual,G1temp,G2temp,adj,
				isat,nmax);
	
	
	/* set isat to 3 for iteration GET_H2O_PROPS calls; future calls will determine
	   properties based on specific enthalpy and density */
	
	isat=3;
	
	
	/* initialize portions of A matrix which are constant */
	
	MXW(mwmat,iup,idown);
	WXP(wpmat,area,len,iup,idown);

	
/************************* start iteration section ***********************************/

do
{
	t=t+dt;

	if(t>=70.5) flag=1; 

	/* calculate the head loss coefficients (including possible two-phase multiplier),
	   pump heads, and heat transfer */

	PUMPHEAD(dppump);
	HEATADDED(heat);
	LCOEF(k,w,dia,area,len,loss,den,rho,iup,qual,press);
	
	
	/* the following determines the real upstream node.  If a link has a negative flow,
	then the downstream node in idown is really the upstream node.  ifrom contains the 
	actual upstream node.  The routines following determine the relevant quantities for
	each link from the true upstream node */
	
	for(i=1;i<=NL;i++)
	{
		if(w[i]<0.0)
			ifrom[i]=idown[i];
		else
			ifrom[i]=iup[i];
	}
	
	
	/* set up A matrices */
	
	WXW(wwmat,k,w);

	if(ifull==0)
	{
		HXW(hwmat,enth,mass,iup,idown,ifrom);
		PXW(pwmat,c1,c2,enth,mass,iup,idown,ifrom);
	}	
	else
	{
		HXW2(hwmat,enth,mass,w,mwmat,dt,iup,idown,ifrom);
		MTRANS(matran,mass,w,dt,iup,idown,ifrom);
		PXW2(pwmat,c1,c2,matran,hwmat,iup,idown);
	}
	
	
	/* set up B matrices */
	
	for(i=1;i<=NL;i++)	bwmat[i]=(area[i]/len[i])*(-dz[i]*den[i]*grav + dppump[i]);
	for(i=1;i<=NN;i++)	bhmat[i]=heat[i];
		
	if(ifull==0)
		for(i=1;i<=NN;i++)		bpmat[i]=c2[i]*heat[i];
	else
	{
		MATMUL(bpmat,matran,bhmat,NN,NN);	
		for(i=1;i<=NN;i++)		bpmat[i]=c2[i]*bpmat[i];
	}

	
	/* convert pressure units to Pa and calculate work vector */

	for(i=1;i<=NN;i++)
	{
		press[i]=press[i]*1.0e6;
		work[i]=press[i]+dt*bpmat[i];
	}


	/* set up A of AX=B (term1) */
	
	for(j=1;j<=NL;j++)
	  for(i=1;i<=NL;i++)
		term1[i][j]=-dt*dt*(wpmat[i][iup[i]]*pwmat[iup[i]][j]+wpmat[i][idown[i]]*
				pwmat[idown[i]][j]);
	
	for(i=1;i<=NL;i++)	term1[i][i]=1-2*dt*wwmat[i][i]+term1[i][i];


	/* set up B of AX=B (term2) */
	
	for(i=1;i<=NL;i++)
		term2[i]=dt*(wpmat[i][iup[i]]*work[iup[i]]+wpmat[i][idown[i]]*work[idown[i]]-
				wwmat[i][i]*w[i]+bwmat[i])+w[i];
	

	/* A (term1) and B (term2) have now been set up */
	/* solve using LU factorization and put solution into flow vector */
	
	LUDCMP(term1,NL,indx,&d);
	LUBKSB(term1,NL,indx,term2);
	for(i=1;i<=NL;i++)	w[i]=term2[i];
	

	/* determine derivative vectors for pressure, mass, and enthalpy */
	
	DxDT(dpdt,w,pwmat,bpmat,NN,NL);
	DxDT(dmdt,w,mwmat,bmmat,NN,NL);

	if(ifull==0)
		DxDT(dhdt,w,hwmat,bhmat,NN,NL);
	else
	{
		DxDT(hold,w,hwmat,bhmat,NN,NL);
		MATMUL(dhdt,matran,hold,NN,NN);
	}
	

	/* explicit update of mass, enthalpy, and pressure */
	
	for(i=1;i<=NN;i++)
	{
		press[i]=(press[i]+dpdt[i]*dt)*1.0e-6;
		dpdt[i]=dpdt[i]*1.0e-6;
		mass[i]=mass[i]+dmdt[i]*dt;
		enth[i]=enth[i]+dhdt[i]*dt;
		spech[i]=enth[i]/mass[i];				
		dtempdt[i]=G1temp[i]*dmdt[i]/vol[i]+G2temp[i]*(dhdt[i]-dmdt[i]*spech[i])/mass[i];
		temp[i]=temp[i]+dtempdt[i]*dt;
		rho[i]=mass[i]/vol[i];
	}		
	

	/* update pressure and temperature; note that isat now equals 3 so that the property 
	   determination is based upon the previously updated values of rho and specific
	   enthalpy (ie pressure and temperature are changed by the GET_H2O_PROPS but mass
	   and enthalpy are not effected */
	
	GET_H2O_PROPS(spech,mass,vol,press,temp,c1,c2,enth,rho,voi,qual,G1temp,G2temp,adj,
			isat,nmax);

	
	/* print if it is time to do so and increment print time value */
	
	if(t>=tprint)	
		PRINTOUT(outfile,w,press,temp,rho,spech,voi,t,dtlast,tfin,&tprint,tp);
	
	
	/* update the time step, dt */
	
	dt=TSTEP(dt,t,&dtlast,dtmin,dtmax,tfin,&tprint,terr,pscale,dpdt);
	

} while (t<tfin);

/************************** end of iteration loop ***********************************/

/* get time upon completion, determine total time elapsed, print to output file,
   and close file */
			
finish=time(&finish);
totaltime=difftime(finish,start);

fprintf(outfile,"actual simulation time: %5.0f seconds",totaltime);

fclose(outfile);
}
	
/************************* end program **********************************************/
